@@ -5,19 +5,25 @@
 import os
 from gettext import gettext as _
 
-from gi.repository import Gio
+from gi.repository import Gio, Gtk
 
 from lutris.command import MonitoredCommand
+from lutris.config import duplicate_game_config
+from lutris.database.games import add_game, get_game_by_field, get_unusued_game_name
 from lutris.game import Game
 from lutris.gui import dialogs
 from lutris.gui.config.add_game import AddGameDialog
 from lutris.gui.config.edit_game import EditGameConfigDialog
+from lutris.gui.dialogs import QuestionDialog
 from lutris.gui.dialogs.log import LogWindow
 from lutris.gui.dialogs.uninstall_game import RemoveGameDialog, UninstallGameDialog
 from lutris.gui.widgets.utils import open_uri
 from lutris.util import xdgshortcuts
-from lutris.util.log import LOG_BUFFERS, logger
+from lutris.util.log import logger
+from lutris.util.steam import shortcut as steam_shortcut
+from lutris.util.strings import gtk_safe
 from lutris.util.system import path_exists
+from lutris.util.wine.dxvk import update_shader_cache
 
 
 class GameActions:
@@ -35,7 +41,6 @@ class GameActions:
             self._game = self.application.get_game_by_id(self.game_id)
             if not self._game:
                 self._game = Game(self.game_id)
-            self._game.connect("game-error", self.window.on_game_error)
         return self._game
 
     @property
@@ -55,13 +60,16 @@ class GameActions:
         return [
             ("play", _("Play"), self.on_game_launch),
             ("stop", _("Stop"), self.on_game_stop),
-            ("show_logs", _("Show logs"), self.on_show_logs),
             ("install", _("Install"), self.on_install_clicked),
+            ("update", _("Install updates"), self.on_update_clicked),
+            ("install_dlcs", "Install DLCs", self.on_install_dlc_clicked),
+            ("show_logs", _("Show logs"), self.on_show_logs),
             ("add", _("Add installed game"), self.on_add_manually),
             ("configure", _("Configure"), self.on_edit_game_configuration),
             ("favorite", _("Add to favorites"), self.on_add_favorite_game),
             ("deletefavorite", _("Remove from favorites"), self.on_delete_favorite_game),
             ("execute-script", _("Execute script"), self.on_execute_script_clicked),
+            ("update-shader-cache", _("Update shader cache"), self.on_update_shader_cache),
             ("browse", _("Browse files"), self.on_browse_files),
             (
                 "desktop-shortcut",
@@ -83,8 +91,19 @@ class GameActions:
                 _("Delete application menu shortcut"),
                 self.on_remove_menu_shortcut,
             ),
+            (
+                "steam-shortcut",
+                _("Create steam shortcut"),
+                self.on_create_steam_shortcut,
+            ),
+            (
+                "rm-steam-shortcut",
+                _("Delete steam shortcut"),
+                self.on_remove_steam_shortcut,
+            ),
             ("install_more", _("Install another version"), self.on_install_clicked),
             ("remove", _("Remove"), self.on_remove_game),
+            ("duplicate", _("Duplicate"), self.on_game_duplicate),
             ("view", _("View on Lutris.net"), self.on_view_game),
             ("hide", _("Hide game from library"), self.on_hide_game),
             ("unhide", _("Unhide game from library"), self.on_unhide_game),
@@ -94,8 +113,12 @@ class GameActions:
         """Return a dictionary of actions that should be shown for a game"""
         return {
             "add": not self.game.is_installed,
+            "duplicate": True,
             "install": not self.game.is_installed,
             "play": self.game.is_installed and not self.is_game_running,
+            "update": self.game.is_updatable,
+            "update-shader-cache": self.game.is_cache_managed,
+            "install_dlcs": self.game.is_updatable,
             "stop": self.is_game_running,
             "configure": bool(self.game.is_installed),
             "browse": self.game.is_installed and self.game.runner_name != "browser",
@@ -103,7 +126,10 @@ class GameActions:
             "favorite": not self.game.is_favorite,
             "deletefavorite": self.game.is_favorite,
             "install_more": not self.game.service and self.game.is_installed,
-            "execute-script": bool(self.game.is_installed and self.game.runner.system_config.get("manual_command")),
+            "execute-script": bool(
+                self.game.is_installed and self.game.runner
+                and self.game.runner.system_config.get("manual_command")
+            ),
             "desktop-shortcut": (
                 self.game.is_installed
                 and not xdgshortcuts.desktop_launcher_exists(self.game.slug, self.game.id)
@@ -112,6 +138,11 @@ class GameActions:
                 self.game.is_installed
                 and not xdgshortcuts.menu_launcher_exists(self.game.slug, self.game.id)
             ),
+            "steam-shortcut": (
+                self.game.is_installed
+                and not steam_shortcut.shortcut_exists(self.game)
+                and not steam_shortcut.is_steam_game(self.game)
+            ),
             "rm-desktop-shortcut": bool(
                 self.game.is_installed
                 and xdgshortcuts.desktop_launcher_exists(self.game.slug, self.game.id)
@@ -120,6 +151,11 @@ class GameActions:
                 self.game.is_installed
                 and xdgshortcuts.menu_launcher_exists(self.game.slug, self.game.id)
             ),
+            "rm-steam-shortcut": bool(
+                self.game.is_installed
+                and steam_shortcut.shortcut_exists(self.game)
+                and not steam_shortcut.is_steam_game(self.game)
+            ),
             "remove": True,
             "view": True,
             "hide": self.game.is_installed and not self.game.is_hidden,
@@ -145,7 +181,7 @@ class GameActions:
 
     def on_show_logs(self, _widget):
         """Display game log"""
-        _buffer = LOG_BUFFERS.get(str(self.game.id))
+        _buffer = self.game.log_buffer
         if not _buffer:
             logger.info("No log for game %s", self.game)
         return LogWindow(
@@ -161,6 +197,15 @@ class GameActions:
             raise RuntimeError("No game to install: %s" % self.game.id)
         self.game.emit("game-install")
 
+    def on_update_clicked(self, _widget):
+        self.game.emit("game-install-update")
+
+    def on_install_dlc_clicked(self, _widget):
+        self.game.emit("game-install-dlc")
+
+    def on_update_shader_cache(self, _widget):
+        update_shader_cache(self.game)
+
     def on_locate_installed_game(self, _button, game):
         """Show the user a dialog to import an existing install to a DRM free service
 
@@ -173,9 +218,43 @@ class GameActions:
         """Callback that presents the Add game dialog"""
         return AddGameDialog(self.window, game=self.game, runner=self.game.runner_name)
 
+    def on_game_duplicate(self, _widget):
+        confirm_dlg = QuestionDialog(
+            {
+                "parent": self.window,
+                "question": _(
+                    "Do you wish to duplicate %s?\nThe configuration will be duplicated, "
+                    "but the games files will <b>not be duplicated</b>."
+                ) % gtk_safe(self.game.name),
+                "title": _("Duplicate game?"),
+            }
+        )
+        if confirm_dlg.result != Gtk.ResponseType.YES:
+            return
+
+        assigned_name = get_unusued_game_name(self.game.name)
+        old_config_id = self.game.game_config_id
+        if old_config_id:
+            new_config_id = duplicate_game_config(self.game.slug, old_config_id)
+        else:
+            new_config_id = None
+
+        db_game = get_game_by_field(self.game.id, "id")
+        db_game["name"] = assigned_name
+        db_game["configpath"] = new_config_id
+        db_game.pop("id")
+        # Disconnect duplicate from service- there should be at most
+        # 1 PGA game for a service game.
+        db_game.pop("service", None)
+        db_game.pop("service_id", None)
+
+        game_id = add_game(**db_game)
+        new_game = Game(game_id)
+        new_game.save()
+
     def on_edit_game_configuration(self, _widget):
         """Edit game preferences"""
-        EditGameConfigDialog(self.window, self.game)
+        self.application.show_window(EditGameConfigDialog, game=self.game, parent=self.window)
 
     def on_add_favorite_game(self, _widget):
         """Add to favorite Games list"""
@@ -218,6 +297,10 @@ class GameActions:
         """Add the selected game to the system's Games menu."""
         xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, menu=True)
 
+    def on_create_steam_shortcut(self, *_args):
+        """Add the selected game to steam as a nonsteam-game."""
+        steam_shortcut.create_shortcut(self.game)
+
     def on_create_desktop_shortcut(self, *_args):
         """Create a desktop launcher for the selected game."""
         xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, desktop=True)
@@ -226,6 +309,10 @@ class GameActions:
         """Remove an XDG menu shortcut"""
         xdgshortcuts.remove_launcher(self.game.slug, self.game.id, menu=True)
 
+    def on_remove_steam_shortcut(self, *_args):
+        """Remove the selected game from list of non-steam apps."""
+        steam_shortcut.remove_shortcut(self.game)
+
     def on_remove_desktop_shortcut(self, *_args):
         """Remove a .desktop shortcut"""
         xdgshortcuts.remove_launcher(self.game.slug, self.game.id, desktop=True)
@@ -237,6 +324,6 @@ class GameActions:
     def on_remove_game(self, *_args):
         """Callback that present the uninstall dialog to the user"""
         if self.game.is_installed:
-            UninstallGameDialog(game_id=self.game.id, parent=self.window)
+            UninstallGameDialog(game_id=self.game.id, parent=self.window).run()
         else:
-            RemoveGameDialog(game_id=self.game.id, parent=self.window)
+            RemoveGameDialog(game_id=self.game.id, parent=self.window).run()
