@@ -3,13 +3,20 @@ import os
 from gettext import gettext as _
 
 import gi
-gi.require_version("WebKit2", "4.0")
+try:
+    gi.require_version("WebKit2", "4.1")
+except (ImportError, ValueError):
+    gi.require_version("WebKit2", "4.0")
+
 from gi.repository import WebKit2
 
-from lutris.gui.dialogs import Dialog
+from lutris.gui.dialogs import ModalDialog
 
 
-class WebConnectDialog(Dialog):
+DEFAULT_USER_AGENT = "Mozilla/5.0 (X11; Linux x86_64; rv:100.0) Gecko/20100101 Firefox/100.0"
+
+
+class WebConnectDialog(ModalDialog):
     """Login form for external services"""
 
     def __init__(self, service, parent=None):
@@ -26,28 +33,37 @@ class WebConnectDialog(Dialog):
         self.service = service
 
         super().__init__(title=service.name, parent=parent)
-        self.set_border_width(0)
-        self.set_default_size(390, 500)
+
+        self.set_default_size(self.service.login_window_width, self.service.login_window_height)
 
         self.webview = WebKit2.WebView.new_with_context(self.context)
         self.webview.load_uri(service.login_url)
         self.webview.connect("load-changed", self.on_navigation)
         self.webview.connect("create", self.on_webview_popup)
+        self.vbox.set_border_width(0)  # pylint: disable=no-member
         self.vbox.pack_start(self.webview, True, True, 0)  # pylint: disable=no-member
 
         webkit_settings = self.webview.get_settings()
+
+        # Set a User Agent
+        webkit_settings.set_user_agent(service.login_user_agent)
+
         # Allow popups (Doesn't work...)
-        webkit_settings.set_enable_write_console_messages_to_stdout(True)
         webkit_settings.set_allow_modal_dialogs(True)
+
         # Enable developer options for troubleshooting (Can be disabled in
         # releases)
-        webkit_settings.set_javascript_can_open_windows_automatically(True)
+        # webkit_settings.set_enable_write_console_messages_to_stdout(True)
+        # webkit_settings.set_javascript_can_open_windows_automatically(True)
         webkit_settings.set_enable_developer_extras(True)
-
+        # self.enable_inspector()
         self.show_all()
 
     def enable_inspector(self):
         """If you want a full blown Webkit inspector, call this"""
+        # WARNING: For some reason this doesn't work as intended.
+        # The inspector shows ups but it's impossible to interact with it
+        # All inputs are blocked by the the webkit dialog.
         inspector = self.webview.get_inspector()
         inspector.show()
 
@@ -78,12 +94,11 @@ class WebConnectDialog(Dialog):
         view = WebKit2.WebView.new_with_related_view(widget)
         view.load_uri(uri)
         popup_dialog = WebPopupDialog(view, parent=self)
-        popup_dialog.set_modal(True)
-        popup_dialog.show()
+        popup_dialog.run()
         return view
 
 
-class WebPopupDialog(Dialog):
+class WebPopupDialog(ModalDialog):
     """Dialog for handling web popups"""
 
     def __init__(self, webview, parent=None):
@@ -96,7 +111,7 @@ class WebPopupDialog(Dialog):
         self.webview.connect("create", self.on_new_webview_popup)
         self.webview.connect("close", self.on_webview_close)
         self.vbox.pack_start(self.webview, True, True, 0)
-        self.set_border_width(0)
+        self.vbox.set_border_width(0)
         self.set_default_size(390, 500)
 
     def on_ready_webview(self, webview):
