@@ -2,7 +2,7 @@
 import array
 import os
 
-from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
+from gi.repository import GdkPixbuf, Gio, GLib, Gtk
 
 from lutris import settings
 from lutris.util import datapath, system
@@ -31,12 +31,7 @@ def get_main_window(widget):
 
 def open_uri(uri):
     """Opens a local or remote URI with the default application"""
-    system.reset_library_preloads()
-    try:
-        Gtk.show_uri(None, uri, Gdk.CURRENT_TIME)
-    except GLib.Error as ex:
-        logger.exception("Failed to open URI %s: %s, falling back to xdg-open", uri, ex)
-        system.execute(["xdg-open", uri])
+    system.spawn(["xdg-open", uri])
 
 
 def get_pixbuf(image, size, fallback=None, is_installed=True):
@@ -54,27 +49,34 @@ def get_pixbuf(image, size, fallback=None, is_installed=True):
             fallback = get_default_icon(size)
         if system.path_exists(fallback):
             pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(fallback, width, height)
-    if is_installed:
+    if is_installed and pixbuf:
         pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.NEAREST)
         return pixbuf
     overlay = os.path.join(datapath.get(), "media/unavailable.png")
     transparent_pixbuf = get_overlay(overlay, size).copy()
-    pixbuf.composite(
-        transparent_pixbuf,
-        0,
-        0,
-        size[0],
-        size[1],
-        0,
-        0,
-        1,
-        1,
-        GdkPixbuf.InterpType.NEAREST,
-        100,
-    )
+    if pixbuf:
+        pixbuf.composite(
+            transparent_pixbuf,
+            0,
+            0,
+            size[0],
+            size[1],
+            0,
+            0,
+            1,
+            1,
+            GdkPixbuf.InterpType.NEAREST,
+            100,
+        )
     return transparent_pixbuf
 
 
+def has_stock_icon(name):
+    """This tests if a GTK stock icon is known; if not we can try a fallback."""
+    theme = Gtk.IconTheme.get_default()
+    return theme.has_icon(name)
+
+
 def get_stock_icon(name, size):
     """Return a pixbuf from a stock icon name"""
     theme = Gtk.IconTheme.get_default()
@@ -85,29 +87,40 @@ def get_stock_icon(name, size):
         return None
 
 
-def get_icon(icon_name, icon_format="image", size=None, icon_type="runner"):
-    """Return an icon based on the given name, format, size and type.
+def get_runtime_icon(icon_name, icon_format="image", size=None):
+    """Return an icon based on the given name, format, size and type. Only
+    the icons installed in Lutris's runtime directory are searched.
 
     Keyword arguments:
     icon_name -- The name of the icon to retrieve
     format -- The format of the icon, which should be either 'image' or 'pixbuf' (default 'image')
     size -- The size for the desired image (default None)
-    icon_type -- Retrieve either a 'runner' or 'platform' icon (default 'runner')
     """
-    filename = icon_name.lower().replace(" ", "") + ".png"
-    icon_path = os.path.join(settings.RUNTIME_DIR, "icons/hicolor/64x64/apps", filename)
-    if not os.path.exists(icon_path):
-        return None
-    if icon_format == "image":
-        icon = Gtk.Image()
-        if size:
-            icon.set_from_pixbuf(get_pixbuf(icon_path, size))
-        else:
-            icon.set_from_file(icon_path)
-        return icon
-    if icon_format == "pixbuf" and size:
-        return get_pixbuf(icon_path, size)
-    raise ValueError("Invalid arguments")
+    filename = icon_name.lower().replace(" ", "")
+    # We prefer bitmaps over SVG, because we've got some SVG icons with the
+    # wrong size (oops) and this avoids them.
+    search_directories = [
+        "icons/hicolor/64x64/apps",
+        "icons/hicolor/24x24/apps",
+        "icons",
+        "icons/hicolor/scalable/apps",
+        "icons/hicolor/symbolic/apps"]
+    extensions = [".png", ".svg"]
+    for search_dir in search_directories:
+        for ext in extensions:
+            icon_path = os.path.join(settings.RUNTIME_DIR, search_dir, filename + ext)
+            if os.path.exists(icon_path):
+                if icon_format == "image":
+                    icon = Gtk.Image()
+                    if size:
+                        icon.set_from_pixbuf(get_pixbuf(icon_path, size))
+                    else:
+                        icon.set_from_file(icon_path)
+                    return icon
+                if icon_format == "pixbuf" and size:
+                    return get_pixbuf(icon_path, size)
+                raise ValueError("Invalid arguments")
+    return None
 
 
 def get_overlay(overlay_path, size):
@@ -123,10 +136,6 @@ def get_default_icon(size):
     return os.path.join(datapath.get(), "media/default_banner.png")
 
 
-def get_pixbuf_for_game(image_abspath, size, is_installed=True):
-    return get_pixbuf(image_abspath, size, fallback=get_default_icon(size), is_installed=is_installed)
-
-
 def convert_to_background(background_path, target_size=(320, 1080)):
     """Converts a image to a pane background"""
     coverart = Image.open(background_path)
