@@ -22,5 +22,5 @@ def migrate():
         return []
     for game_id in game_ids:
         game = Game(game_id)
-        game.hide()
+        game.set_hidden(True)
     settings.write_setting("library_ignores", '', section="lutris")
