@@ -35,7 +35,12 @@ class yuzu(Runner):
             "label": _("Title keys"),
             "type": "file",
             "help": _("File containing the title keys."),
-        }
+        }, {
+            "option": "fullscreen",
+            "label": _("Fullscreen"),
+            "type": "bool",
+            "default": True,
+        },
     ]
 
     @property
@@ -44,16 +49,21 @@ class yuzu(Runner):
         candidates = ("~/.local/share/yuzu", )
         for candidate in candidates:
             path = system.fix_path_case(os.path.join(os.path.expanduser(candidate), "nand"))
-            if path:
+            if path and system.path_exists(path):
                 return path[:-len("nand")]
 
     def play(self):
         """Run the game."""
         arguments = [self.get_executable()]
+
+        fullscreen = self.runner_config.get("fullscreen")
+        if fullscreen:
+            arguments.append("-f")
+
         rom = self.game_config.get("main_file") or ""
         if not system.path_exists(rom):
             return {"error": "FILE_NOT_FOUND", "file": rom}
-        arguments.append(rom)
+        arguments += ["-g", rom]
         return {"command": arguments}
 
     def _update_key(self, key_type):
@@ -89,4 +99,3 @@ class yuzu(Runner):
     def prelaunch(self):
         for key in ["prod_keys", "title_keys"]:
             self._update_key(key_type=key)
-        return True
