@@ -6,7 +6,7 @@ import time
 from gi.repository import GLib
 
 from lutris import settings
-from lutris.util import http, jobs, system
+from lutris.util import http, jobs, system, update_cache
 from lutris.util.downloader import Downloader
 from lutris.util.extract import extract_archive
 from lutris.util.linux import LINUX_SYSTEM
@@ -187,12 +187,28 @@ class Runtime:
 class RuntimeUpdater:
     """Class handling the runtime updates"""
 
-    current_updates = 0
+    cancelled = False
     status_updater = None
+    downloaders = {}
+
+    def __init__(self, force=False):
+        self.force = force
+
+    def update_runtimes(self):
+        """Update runtime components"""
+        runtime_call = update_cache.get_last_call("runtime")
+        if self.force or not runtime_call or runtime_call > 3600 * 12:
+            components_to_update = self.update()
+            if components_to_update:
+                while self.downloaders:
+                    time.sleep(0.3)
+                    if self.cancelled:
+                        return
+            update_cache.write_date_to_cache("runtime")
 
     def is_updating(self):
         """Return True if the update process is running"""
-        return self.current_updates > 0
+        return bool(self.downloaders)
 
     def update(self):
         """Launch the update process"""
@@ -204,8 +220,13 @@ class RuntimeUpdater:
             runtime = Runtime(remote_runtime["name"], self)
             downloader = runtime.download(remote_runtime)
             if downloader:
-                self.current_updates += 1
-        return self.current_updates
+                self.downloaders[runtime] = downloader
+        return len(self.downloaders)
+
+    def cancel(self):
+        self.cancelled = True
+        for downloader in self.downloaders:
+            downloader.cancel()
 
     @staticmethod
     def _iter_remote_runtimes():
@@ -244,8 +265,8 @@ class RuntimeUpdater:
     def notify_finish(self, runtime):
         """A runtime has finished downloading"""
         logger.debug("Runtime %s is now updated and available", runtime.name)
-        self.current_updates -= 1
-        if self.current_updates == 0:
+        del self.downloaders[runtime]
+        if not self.downloaders:
             logger.info("Runtime is fully updated.")
 
 
@@ -261,13 +282,13 @@ def get_env(version=None, prefer_system_libs=False, wine_path=None):
     Returns:
         dict
     """
-    return {
-        key: value
-        for key, value in {
-            "LD_LIBRARY_PATH":
-            ":".join(get_paths(version=version, prefer_system_libs=prefer_system_libs, wine_path=wine_path)),
-        }.items() if value
-    }
+    library_path = ":".join(get_paths(version=version, prefer_system_libs=prefer_system_libs, wine_path=wine_path))
+    env = {}
+    if library_path:
+        env["LD_LIBRARY_PATH"] = library_path
+        network_tools_path = os.path.join(settings.RUNTIME_DIR, "network-tools")
+        env["PATH"] = "%s:%s" % (network_tools_path, os.environ["PATH"])
+    return env
 
 
 def get_winelib_paths(wine_path):
