@@ -8,18 +8,17 @@ from urllib.parse import parse_qsl, urlencode, urlparse
 
 from lxml import etree
 
-import lutris.util.i18n as i18n
 from lutris import settings
-from lutris.exceptions import AuthenticationError, UnavailableGame
+from lutris.exceptions import AuthenticationError, UnavailableGameError
 from lutris.installer import AUTO_ELF_EXE, AUTO_WIN32_EXE
 from lutris.installer.installer_file import InstallerFile
 from lutris.services.base import OnlineService
 from lutris.services.service_game import ServiceGame
 from lutris.services.service_media import ServiceMedia
-from lutris.util import system
+from lutris.util import i18n, system
 from lutris.util.http import HTTPError, Request, UnauthorizedAccess
 from lutris.util.log import logger
-from lutris.util.strings import slugify
+from lutris.util.strings import human_size, slugify
 
 
 class GogSmallBanner(ServiceMedia):
@@ -28,6 +27,7 @@ class GogSmallBanner(ServiceMedia):
     size = (100, 60)
     dest_path = os.path.join(settings.CACHE_DIR, "gog/banners/small")
     file_pattern = "%s.jpg"
+    file_format = "jpeg"
     api_field = "image"
     url_pattern = "https:%s_prof_game_100x60.jpg"
 
@@ -178,14 +178,15 @@ class GOGService(OnlineService):
         request = Request(url)
         try:
             request.get()
-        except HTTPError:
+        except HTTPError as http_error:
+            logger.error(http_error)
             logger.error("Failed to get token, check your GOG credentials.")
             logger.warning("Clearing existing credentials")
             self.logout()
             return
 
         token = request.json
-        with open(self.token_path, "w") as token_file:
+        with open(self.token_path, "w", encoding='utf-8') as token_file:
             token_file.write(json.dumps(token))
         if not refresh_token:
             self.emit("service-login")
@@ -194,7 +195,7 @@ class GOGService(OnlineService):
         """Load token from disk"""
         if not os.path.exists(self.token_path):
             raise AuthenticationError("No GOG token available")
-        with open(self.token_path) as token_file:
+        with open(self.token_path, encoding='utf-8') as token_file:
             token_content = json.loads(token_file.read())
         return token_content
 
@@ -208,6 +209,8 @@ class GOGService(OnlineService):
         """Send a cookie authenticated HTTP request to GOG"""
         request = Request(url, cookies=self.load_cookies())
         request.get()
+        if request.content.startswith(b"<"):
+            raise AuthenticationError("Token expired, please log in again")
         return request.json
 
     def make_api_request(self, url):
@@ -246,7 +249,7 @@ class GOGService(OnlineService):
         """Return the user's library of GOG games"""
         if system.path_exists(self.cache_path):
             logger.debug("Returning cached GOG library")
-            with open(self.cache_path, "r") as gog_cache:
+            with open(self.cache_path, "r", encoding='utf-8') as gog_cache:
                 return json.load(gog_cache)
 
         total_pages = 1
@@ -257,7 +260,7 @@ class GOGService(OnlineService):
             page += 1
             total_pages = products_response["totalPages"]
             games += products_response["products"]
-        with open(self.cache_path, "w") as gog_cache:
+        with open(self.cache_path, "w", encoding='utf-8') as gog_cache:
             json.dump(games, gog_cache)
         return games
 
@@ -276,6 +279,14 @@ class GOGService(OnlineService):
         url = self.embed_url + "/account/getFilteredProducts?" + urlencode(params)
         return self.make_request(url)
 
+    def get_game_dlcs(self, product_id):
+        """Return the list of DLC products for a game"""
+        game_details = self.get_game_details(product_id)
+        if not game_details["dlcs"]:
+            return []
+        all_products_url = game_details["dlcs"]["expanded_all_products_url"]
+        return self.make_api_request(all_products_url)
+
     def get_game_details(self, product_id):
         """Return game information for a given game"""
         if not product_id:
@@ -291,9 +302,9 @@ class GOGService(OnlineService):
             response = self.make_api_request(downlink)
         except HTTPError as ex:
             logger.error("HTTP error: %s", ex)
-            raise UnavailableGame
+            raise UnavailableGameError(_("The download of '%s' failed.") % downlink) from ex
         if not response:
-            raise UnavailableGame
+            raise UnavailableGameError(_("The download of '%s' failed.") % downlink)
         for field in ("checksum", "downlink"):
             field_url = response[field]
             parsed = urlparse(field_url)
@@ -303,45 +314,67 @@ class GOGService(OnlineService):
 
     def get_downloads(self, gogid):
         """Return all available downloads for a GOG ID"""
+        if not gogid:
+            logger.warning("Unable to get GOG data because no GOG ID is available")
+            return {}
         gog_data = self.get_game_details(gogid)
         if not gog_data:
             logger.warning("Unable to get GOG data for game %s", gogid)
-            return []
+            return {}
         return gog_data["downloads"]
 
     def get_extras(self, gogid):
-        """Return a list of bonus content available for a GOG ID"""
-        downloads = self.get_downloads(gogid)
-        return [
-            {
-                "name": download.get("name", ""),
-                "type": download.get("type", ""),
-                "total_size": download.get("total_size", 0),
-                "id": str(download["id"]),
-            } for download in downloads.get("bonus_content") or []
-        ]
+        """Return a list of bonus content available for a GOG ID and its DLCs"""
+        logger.debug("Download extras for GOG ID %s and its DLCs", gogid)
+        game = self.get_game_details(gogid)
+        if not game:
+            logger.warning("Unable to get GOG data for game %s", gogid)
+            return []
+        dlcs = self.get_game_dlcs(gogid)
+        products = [game, *dlcs] if dlcs else [game]
+        all_extras = {}
+        for product in products:
+            extras = [
+                {
+                    "name": download.get("name", "").strip().capitalize(),
+                    "type": download.get("type", "").strip(),
+                    "total_size": download.get("total_size", 0),
+                    "id": str(download["id"]),
+                } for download in product["downloads"].get("bonus_content") or []
+            ]
+            if extras:
+                all_extras[product.get("title", "").strip()] = extras
+        return all_extras
 
     def get_installers(self, downloads, runner, language="en"):
         """Return available installers for a GOG game"""
-
         # Filter out Mac installers
-        gog_installers = [installer for installer in downloads["installers"] if installer["os"] != "mac"]
+        gog_installers = [installer for installer in downloads.get("installers", []) if installer["os"] != "mac"]
         available_platforms = {installer["os"] for installer in gog_installers}
         # If it's a Linux game, also filter out Windows games
         if "linux" in available_platforms:
-            if runner == "linux":
-                filter_os = "windows"
-            else:
-                filter_os = "linux"
+            filter_os = "windows" if runner == "linux" else "linux"
             gog_installers = [installer for installer in gog_installers if installer["os"] != filter_os]
+        return [
+            installer
+            for installer in gog_installers
+            if installer["language"] == self.determine_language_installer(gog_installers, language)
+        ]
 
-        language = self.determine_language_installer(gog_installers, language)
-        gog_installers = [installer for installer in gog_installers if installer["language"] == language]
-        return gog_installers
+    def get_update_versions(self, gog_id):
+        """Return updates available for a game, keyed by patch version"""
+        games_detail = self.get_game_details(gog_id)
+        patches = games_detail["downloads"]["patches"]
+        if not patches:
+            logger.info("No patches for %s", games_detail)
+            return {}
+        patch_versions = defaultdict(list)
+        for patch in patches:
+            patch_versions[patch["name"]].append(patch)
+        return patch_versions
 
-    def determine_language_installer(self, gog_installers, default_language):
+    def determine_language_installer(self, gog_installers, default_language="en"):
         """Return locale language string if available in gog_installers"""
-
         language = i18n.get_lang()
         gog_installers = [installer for installer in gog_installers if installer["language"] == language]
         if not gog_installers:
@@ -388,23 +421,36 @@ class GOGService(OnlineService):
                 )
         return extra_files
 
-    def get_installer_files(self, installer, installer_file_id):
+    def _get_installer_links(self, installer, downloads):
+        """Return links to downloadable files from a list of downloads"""
         try:
-            downloads = self.get_downloads(installer.service_appid)
             gog_installers = self.get_installers(downloads, installer.runner)
             if not gog_installers:
                 return []
             if len(gog_installers) > 1:
                 logger.warning("More than 1 GOG installer found, picking first.")
             _installer = gog_installers[0]
-            links = self.query_download_links(_installer)
-        except HTTPError:
-            raise UnavailableGame("Couldn't load the download links for this game")
-        if not links:
-            raise UnavailableGame("Could not fing GOG game")
+            return self.query_download_links(_installer)
+        except HTTPError as err:
+            raise UnavailableGameError(_("Couldn't load the download links for this game")) from err
+
+    def get_patch_files(self, installer, installer_file_id):
+        logger.debug("Getting patches for %s", installer.version)
+        downloads = self.get_downloads(installer.service_appid)
+        links = []
+        for patch_file in downloads["patches"]:
+            if "GOG " + patch_file["version"] == installer.version:
+                links += self.query_download_links(patch_file)
+        return self._format_links(installer, installer_file_id, links)
+
+    def _format_links(self, installer, installer_file_id, links):
         _installer_files = defaultdict(dict)  # keyed by filename
         for link in links:
-            filename = link["filename"]
+            try:
+                filename = link["filename"]
+            except KeyError:
+                logger.error("Invalid link: %s", link)
+                raise
             if filename.lower().endswith(".xml"):
                 if filename != installer_file_id:
                     filename = filename[:-4]
@@ -432,7 +478,19 @@ class GOGService(OnlineService):
                 "checksum_url": installer_file.get("checksum_url")
             }))
         if not file_id_provided:
-            raise UnavailableGame("Unable to determine correct file to launch installer")
+            raise UnavailableGameError(_("Unable to determine correct file to launch installer"))
+        return files
+
+    def get_installer_files(self, installer, installer_file_id):
+        try:
+            downloads = self.get_downloads(installer.service_appid)
+        except HTTPError as err:
+            raise UnavailableGameError(_("Couldn't load the downloads for this game")) from err
+        links = self._get_installer_links(installer, downloads)
+        if links:
+            files = self._format_links(installer, installer_file_id, links)
+        else:
+            files = []
         if self.selected_extras:
             for extra_file in self.get_extra_files(downloads, installer):
                 files.append(extra_file)
@@ -445,7 +503,7 @@ class GOGService(OnlineService):
         """
         if not file_path.endswith(".xml"):
             raise ValueError("Pass a XML file to return the checksum")
-        with open(file_path) as checksum_file:
+        with open(file_path, encoding='utf-8') as checksum_file:
             checksum_content = checksum_file.read()
         root_elem = etree.fromstring(checksum_content)
         return (root_elem.attrib["name"], root_elem.attrib["md5"])
@@ -483,3 +541,110 @@ class GOGService(OnlineService):
                 "installer": script
             }
         }
+
+    def get_games_owned(self):
+        """Return IDs of games owned by user"""
+        url = "{}/user/data/games".format(self.embed_url)
+        return self.make_api_request(url)
+
+    def get_dlc_installers(self, db_game):
+        """Return all available DLC installers for game"""
+        appid = db_game["service_id"]
+
+        dlcs = self.get_game_dlcs(appid)
+
+        installers = []
+
+        for dlc in dlcs:
+            dlc_id = "gogdlc-%s" % dlc["slug"]
+
+            # remove mac installers for now
+            installfiles = [installer for installer in dlc["downloads"].get(
+                "installers", []) if installer["os"] != "mac"]
+
+            for file in installfiles:
+                # supports linux
+                if file["os"].lower() == "linux":
+                    runner = "linux"
+                    script = [{"extract": {"dst": "$CACHE/GOG", "file": dlc_id, "format": "zip"}},
+                              {"merge": {"dst": "$GAMEDIR", "src": "$CACHE/GOG/data/noarch/"}}]
+                else:
+                    runner = "wine"
+                    script = [{"task": {"name": "wineexec", "executable": dlc_id}}]
+
+                installer = {
+                    "name": db_game["name"],
+                    # add runner in brackets - wrong installer can be run when this is not unique
+                    "version": f"{dlc['title']} ({runner})",
+                    "slug": dlc["slug"],
+                    "description": "DLC for %s" % db_game["name"],
+                    "game_slug": slugify(db_game["name"]),
+                    "runner": runner,
+                    "is_dlc": True,
+                    "dlcid": dlc["id"],
+                    "gogid": dlc["id"],
+                    "script": {
+                        "extends": db_game["installer_slug"],
+                        "files": [
+                            {dlc_id: "N/A:Select the patch from GOG"}
+                        ],
+                        "installer": script
+                    }
+                }
+                installers.append(installer)
+
+        return installers
+
+    def get_dlc_installers_owned(self, db_game):
+        """Return DLC installers for owned DLC"""
+
+        owned = self.get_games_owned()
+        installers = self.get_dlc_installers(db_game)
+
+        installers = [installer for installer in installers if installer["dlcid"] in owned["owned"]]
+
+        return installers
+
+    def get_dlc_installers_runner(self, db_game, runner, only_owned=True):
+        """Return DLC installers for requested runner
+        only_owned=True only return installers for owned DLC (default)"""
+        if only_owned:
+            installers = self.get_dlc_installers_owned(db_game)
+        else:
+            installers = self.get_dlc_installers(db_game)
+
+        # only handle linux & wine for now
+        if runner != "linux":
+            runner = "wine"
+
+        installers = [installer for installer in installers if installer["runner"] == runner]
+
+        return installers
+
+    def get_update_installers(self, db_game):
+        appid = db_game["service_id"]
+        patch_versions = self.get_update_versions(appid)
+        patch_installers = []
+        for version in patch_versions:
+            patch = patch_versions[version]
+            size = human_size(sum(part["total_size"] for part in patch))
+            patch_id = "gogpatch-%s" % slugify(patch[0]["version"])
+            installer = {
+                "name": db_game["name"],
+                "description": patch[0]["name"] + " " + size,
+                "slug": db_game["installer_slug"],
+                "game_slug": db_game["slug"],
+                "version": "GOG " + patch[0]["version"],
+                "runner": "wine",
+                "script": {
+                    "extends": db_game["installer_slug"],
+                    "files": [
+                        {patch_id: "N/A:Select the patch from GOG"}
+                    ],
+                    "installer": [
+                        {"task": {"name": "wineexec", "executable": patch_id}}
+                    ]
+                }
+            }
+            patch_installers.append(installer)
+        return patch_installers
