@@ -67,7 +67,7 @@ class Downloader:
         self.last_check_time = get_time()
         if self.overwrite and os.path.isfile(self.dest):
             os.remove(self.dest)
-        self.file_pointer = open(self.dest, "wb")
+        self.file_pointer = open(self.dest, "wb")  # pylint: disable=consider-using-with
         self.thread = jobs.AsyncCall(self.async_download, self.download_cb)
         self.stop_request = self.thread.stop_request
 
@@ -140,7 +140,7 @@ class Downloader:
         headers["User-Agent"] = "Lutris/%s" % __version__
         if self.referer:
             headers["Referer"] = self.referer
-        response = requests.get(self.url, headers=headers, stream=True)
+        response = requests.get(self.url, headers=headers, stream=True, timeout=30)
         if response.status_code != 200:
             logger.info("%s returned a %s error", self.url, response.status_code)
         response.raise_for_status()
