@@ -1,12 +1,10 @@
-"""Steam appmanifest file hnadling"""
-# Standard Library
+"""Steam appmanifest file handling"""
 import os
 import re
 
-# Lutris Modules
 from lutris.util.log import logger
 from lutris.util.steam.config import get_steamapps_paths
-from lutris.util.steam.vdf import vdf_parse
+from lutris.util.steam.vdfutils import vdf_parse
 from lutris.util.strings import slugify
 from lutris.util.system import fix_path_case, path_exists
 
@@ -46,7 +44,7 @@ class AppManifest:
         self.appmanifest_data = {}
 
         if path_exists(appmanifest_path):
-            with open(appmanifest_path, "r") as appmanifest_file:
+            with open(appmanifest_path, "r", encoding='utf-8') as appmanifest_file:
                 self.appmanifest_data = vdf_parse(appmanifest_file, {})
         else:
             logger.error("Path to AppManifest file %s doesn't exist", appmanifest_path)
@@ -102,9 +100,8 @@ class AppManifest:
         if not self.installdir:
             return None
         install_path = fix_path_case(os.path.join(self.steamapps_path, "common", self.installdir))
-        if install_path:
+        if install_path and path_exists(install_path):
             return install_path
-
         return None
 
     def get_platform(self):
@@ -116,10 +113,6 @@ class AppManifest:
             return "windows"
         raise ValueError("Can't find %s in %s" % (self.steamapps_path, steamapps_paths))
 
-    def get_runner_name(self):
-        """Runner used by the Steam game"""
-        return "steam" if self.get_platform() == "linux" else "winesteam"
-
 
 def get_appmanifest_from_appid(steamapps_path, appid):
     """Given the steam apps path and appid, return the corresponding appmanifest"""
