@@ -13,7 +13,7 @@ def read_yaml_from_file(filename):
     if not path_exists(filename):
         return {}
 
-    with open(filename, "r") as yaml_file:
+    with open(filename, "r", encoding='utf-8') as yaml_file:
         try:
             yaml_content = yaml.safe_load(yaml_file) or {}
         except (yaml.scanner.ScannerError, yaml.parser.ParserError):
@@ -24,6 +24,6 @@ def read_yaml_from_file(filename):
 
 
 def write_yaml_to_file(config, filepath):
-    yaml_config = yaml.dump(config, default_flow_style=False)
-    with open(filepath, "w") as filehandler:
+    yaml_config = yaml.safe_dump(config, default_flow_style=False)
+    with open(filepath, "w", encoding='utf-8') as filehandler:
         filehandler.write(yaml_config)
